@@ -3,6 +3,7 @@
 // Feel free to use/distribute/modify as long as credit/copyrights for myself 
 // are included.
 
+#include <math.h>
 #include "FastMath.h"
 #include "iostream.h"
 
@@ -11,7 +12,7 @@ double FastMath::Vsin[TRIGSIZE+1];
 FixedPoint FastMath::VFcos[TRIGSIZE+1];
 FixedPoint FastMath::VFsin[TRIGSIZE+1];
 
-//unsigned char FastMath::VintSqrt[SQRTSIZE];
+unsigned char FastMath::VintSqrt[SQRTSIZE];
 
 void FastMath::init(int verbose)
 	{
@@ -35,12 +36,12 @@ void FastMath::init(int verbose)
 			if ( (i%6)==0 ) cout << '.';
 		}
 
-#if 0
+//#if 0
 	for(i=0; i<SQRTSIZE; i++)
 		{
 		VintSqrt[i] = (unsigned char)sqrt(i);
 		}
-#endif
+//#endif
 
 	if (verbose) cout << endl;
 	}
