@@ -10,8 +10,7 @@
 // Note: no error checking on format of save file!
 HiScore::HiScore( const char *file )
 	{
-	Vfile = SCOREDIR;
-	Vfile += file;
+	Vfile = file;
 
 	// init list
 	int i;
