@@ -16,12 +16,6 @@
 
 #define HiScoreListSize	10
 #define MaxNameLength	20
-// SCOREDIR determines where the high-score file will be stored.
-// If not defined in the makefile, default it to the current directory
-#ifndef SCOREDIR
-#define SCOREDIR "./"
-#endif
-
 
 typedef unsigned int Score;
 
