@@ -14,7 +14,6 @@
 
 #include "SBitmap.h"
 #include "ObjTools.h"
-#include "Explosion.h"
 #include "FastMath.h"
 
 //////////////////////////////////////////////////////////////////////////////
@@ -46,6 +45,11 @@ class Sprite : public Life, public Trajectory {
 		Vbitmap->put( FP2INT(VFx)-Vsize, FP2INT(VFy)-Vsize );
     		}
 
+    	inline virtual void intDraw( int x, int y)
+    		{
+		Vbitmap->put( x-Vsize, y-Vsize );
+    		}
+
 
 	// do the stuff that should happen in one 'frame' of time, i.e.
 	// move the obj, etc..
@@ -56,16 +60,10 @@ class Sprite : public Life, public Trajectory {
 		Trajectory::move();
 		}
 
-	inline void explode(int size, int dur)
-		{
-		// add an explosion to the explosion list
-		Boom::add( VFx, VFy, size, dur, VFxvel, VFyvel );
-		}
-
 	inline void setBitmap( const SBitmap &b );
 
 	//Bitmap& bitmap() { return b; }
-	
+
 	inline int	size() const { return Vsize; }
 
 	inline void	towards( const Sprite& b, FixedPoint Fspeed )
@@ -82,7 +80,7 @@ class Sprite : public Life, public Trajectory {
 	FixedPoint Fmass;
 
 	
-    private:
+    protected:
 	const SBitmap *Vbitmap;
 	int		Vsize;
      };
