@@ -9,6 +9,7 @@
 // string input from the user, etc..
 
 #include "Ui.h"
+#include "Keyboard.h"
 
 GraphicsContext Ui::backscreen;
 GraphicsContext Ui::physicalscreen;
@@ -58,19 +59,19 @@ void Ui::init()
 
 	// trap signals to restore text mode, etc
 	signal(SIGHUP, die);
-	signal(SIGINT, die);
+	//signal(SIGINT, die);
 	signal(SIGQUIT, die);
 	signal(SIGILL, die);
-	signal(SIGTRAP, die);
+	//signal(SIGTRAP, die);
 	signal(SIGABRT, die);
 	signal(SIGFPE, die);
 	signal(SIGKILL, die);
 	signal(SIGSEGV, die);
 	signal(SIGTERM, die);
-	signal(SIGSTOP, die);
-	signal(SIGTSTP, die);
-	signal(SIGTTIN, die);
-	signal(SIGTTOU, die);
+	//signal(SIGSTOP, die);
+	//signal(SIGTSTP, die);
+	//signal(SIGTTIN, die);
+	//signal(SIGTTOU, die);
 
 		
 	vga_setmode(vgaMode);
@@ -92,18 +93,16 @@ void Ui::init()
 	}
 
 
-
-
 void Ui::die(int x)
 	{
 	Ui::restore();
 
 	cerr << "Ui::Received signal " << x << endl;
-
+	
+	Keyboard::restore();	// in case we are in raw keyboard mode
 	exit(-1);
 	}
 
-
 void Ui::restore()
 	{
 
