@@ -23,8 +23,6 @@
 
 #include "str.h"
 
-unsigned char gamefont01[2048];
-unsigned char gamefont02[2048];
 
 class Ui {
 
@@ -78,6 +76,10 @@ class Ui {
 	static unsigned char *VexpandedFont;
   
   	static struct termios old;
+
+	static unsigned char gamefont01[2048];
+	static unsigned char gamefont02[2048];
+
     };
 
 
