@@ -19,17 +19,14 @@
 #include "cfclass.h"
 #include "Keyboard.h"
 #include "Timer.h"
-#include "sasteroids.h"
 #include "HiScore.h"
 #include "Ui.h"
-#include "Obj.h"		// game sprites
 #include "FastMath.h"
 #include "Explosion.h"
 #include "Joystick.h"
 
-
-// not ready yet
-//#include "Sound.h"
+#include "Obj.h"		// game sprites
+#include "sasteroids.h"
 
 // Version should be defined and passed in from Makefile
 #ifndef VERSION
@@ -71,12 +68,12 @@
 // to work the way they do..
  
 // Masses	
-#define M_BIG	INT2FP(8)
-#define M_MED	INT2FP(4)
-#define M_SMALL	INT2FP(1)
-#define M_SHIP	INT2FP(5)>>2
-#define M_ENEMY	INT2FP(1)
-#define M_BULLET INT2FP(1)>>3
+#define M_BIG	FLOAT2FP(8.0)
+#define M_MED	FLOAT2FP(4.0)
+#define M_SMALL	FLOAT2FP(1.0)
+#define M_SHIP	FLOAT2FP(5.0/4.0 )
+#define M_ENEMY	FLOAT2FP(1.0)
+#define M_BULLET FLOAT2FP( (1.0/8.0) )
 
 #define START_DIST 70		// when starting a level, place new asteroids
 				// at least this far from your ship.
@@ -98,7 +95,10 @@ Ship	playerShip( SHIPSHAPE, M_SHIP );
 // A cfc object provides an interface to '.cf' files
 cfc	Gcf;
 
-// some bitmaps needed on-the-fly through the game
+// manages the explosions:
+BoomManager<SimpleExpl> GboomList;
+
+// some bitmaps needed on-the-fly through-out the game
 SBitmap Gbit[8];
 SBitmap Gbackdrop;
 
@@ -218,18 +218,15 @@ void makeasts()
 		{
 		obj[i]->shape = ASTSHAPE1;
 
-		obj[i]->setFpos( INT2FP((rand()>>5)%WIDTH), 
-		                INT2FP((rand()>>5)%HEIGHT) );
-
-		// make sure new ast is not near player's ship
-		if (distance( playerShip, *obj[i] ) > START_DIST)
-			{
-			obj[i]->randomDir( INT2FP(1) );
-			obj[i]->setBitmap(Gbit[BIGAST]);
-			obj[i]->Fmass = M_BIG;
-			obj[i]->revive();
-			}
-		else	i--;	// ast too close to player, try again
+		do {
+		   obj[i]->setFpos( INT2FP((rand()>>5)%WIDTH), 
+		                    INT2FP((rand()>>5)%HEIGHT) );
+		   } while (distance( playerShip, *obj[i] ) < START_DIST);
+		   
+		obj[i]->randomDir( INT2FP(1) );
+		obj[i]->setBitmap(Gbit[BIGAST]);
+		obj[i]->Fmass = M_BIG;
+		obj[i]->revive();
 		}
 	numasts = i;
 	}
@@ -279,7 +276,9 @@ void bounce(int i, int j )
 	ij.x = obj[j]->VFx - obj[i]->VFx; ij.y = obj[j]->VFy - obj[i]->VFy;
 	ij.mag = FastMath::intSqrt( FP2INT(ij.x) * FP2INT(ij.x) + 
 	                    FP2INT(ij.y) * FP2INT(ij.y) );
-	
+
+	if (ij.mag==0) return;
+		
 	vi_ij.mag = dot(vi, ij) / ij.mag;
 	vi_ij.x = FPMULT(ij.x , vi_ij.mag) / ij.mag ;
 	vi_ij.y = FPMULT(ij.y , vi_ij.mag) / ij.mag ;
@@ -303,8 +302,9 @@ void bounce(int i, int j )
 	vi_ijf.x = vj_ijf.x + vj_ij.x - vi_ij.x;
 	vi_ijf.y = vj_ijf.y + vj_ij.y - vi_ij.y;
 
-	obj[i]->setFvel( vi_ijf.x + vipij.x, vi_ijf.y + vipij.y );
-	obj[j]->setFvel( vj_ijf.x + vjpij.x, vj_ijf.y + vjpij.y );
+	// clip to -10...10 max velocity in either x or y direction.
+	obj[i]->setFvel( ((vi_ijf.x + vipij.x) >? -INT2FP(10) ) <? INT2FP(10), ((vi_ijf.y + vipij.y) >? -INT2FP(10) ) <? INT2FP(10) );
+	obj[j]->setFvel( ((vj_ijf.x + vjpij.x) >? -INT2FP(10) ) <? INT2FP(10), ((vj_ijf.y + vjpij.y) >? -INT2FP(10) ) <? INT2FP(10) );
 
 	}
 
@@ -363,42 +363,46 @@ void killast(int killer, int i)
 	// i = Asteroid # to kill	
 	{	
 	int k=0, na=0, oldna=0;
-
-	if (obj[i]->shape == ASTSHAPE1)
+	ObjPtr o1=NULL,o2=NULL;
+	
+	o1 = obj[i];
+	if (o1->shape == ASTSHAPE1)
 		{	
 		if (numasts == 1)	// Big asteroid was last remaining 
 			upscore(killer, LEVEL * 1000);
 		else	upscore(killer, 25);
 		
-		na = nextast();
-		obj[na]->setFpos( obj[i]->VFx, obj[i]->VFy );
-		obj[na]->setFvel( obj[i]->VFxvel>>1, obj[i]->VFyvel>>1 );
-
-		obj[na]->revive();
-		obj[na]->shape = ASTSHAPE2;
-		obj[na]->setBitmap(Gbit[MEDAST]);
-		obj[na]->Fmass = M_MED;
+		na=nextast();
+		o2=obj[na];
 		
-	// every once in a while, one of the medium asteroids is replaced
-	// with a little devil ship that flies around and
-	// tries to crash into you..
+		o2->setFpos( o1->VFx, o1->VFy );
+		o2->setFvel( o1->VFxvel>>1, o1->VFyvel>>1 );
+
+		o2->revive();
+		o2->shape = ASTSHAPE2;
+		o2->setBitmap(Gbit[MEDAST]);
+		o2->Fmass = M_MED;
+		
+		// every once in a while, one of the medium asteroids is replaced
+		// with a little devil ship that flies around and
+		// tries to crash into you..
 		if (((rand()>>4)%100)<Glevel)
 			{
-			obj[i] = new Spinner( SPINNERSHAPE, M_BIG, playerShip );
-			((Spinner *)obj[i])->newShip();
-			((Spinner *)obj[i])->setFpos( obj[na]->VFx, obj[na]->VFy );
+			o1 = obj[i] = new Spinner( SPINNERSHAPE, M_BIG, playerShip );
+			((Spinner *)o1)->newShip();
+			((Spinner *)o1)->setFpos( o2->VFx, o2->VFy );
 			}
 		else
 			{
-			obj[i]->shape = ASTSHAPE2;
-			obj[i]->setBitmap(Gbit[MEDAST]);
-			obj[i]->Fmass = M_MED;
+			o1->shape = ASTSHAPE2;
+			o1->setBitmap(Gbit[MEDAST]);
+			o1->Fmass = M_MED;
 			numasts++;
 			}
 		blastpair(i, na);
-		obj[i]->explode(30,12);
+		GboomList.add( o1->VFx, o1->VFy, 30, 12, o1->VFxvel, o1->VFyvel);
 		}
-	else if (obj[i]->shape == ASTSHAPE2)
+	else if (o1->shape == ASTSHAPE2)
 		{
 		if (numasts == 1) upscore(killer, 500*LEVEL);
 
@@ -406,45 +410,47 @@ void killast(int killer, int i)
 			{	
 			oldna = na;
 			na = nextast();
-			obj[na]->setFpos( obj[i]->VFx, obj[i]->VFy );
-			obj[na]->setFvel( obj[i]->VFxvel, obj[i]->VFyvel );
+			o2 = obj[na];
+			o2->setFpos( o1->VFx, o1->VFy );
+			o2->setFvel( o1->VFxvel, o1->VFyvel );
 
-			obj[na]->revive();
-			obj[na]->shape = ASTSHAPE3;
-			obj[na]->setBitmap(Gbit[SMALLAST]);
-			obj[na]->Fmass = M_SMALL;
+			o2->revive();
+			o2->shape = ASTSHAPE3;
+			o2->setBitmap(Gbit[SMALLAST]);
+			o2->Fmass = M_SMALL;
 			if (k == 1) blastpair(oldna,na);
 			}
 		if (((rand()>>4)&511)<(Glevel+10))
 			{
-			obj[i] = new Spinner( SPINNERSHAPE, M_BIG, playerShip );
-			((Spinner *)obj[i])->newShip();
-			((Spinner *)obj[i])->setFpos( obj[na]->VFx, obj[na]->VFy );
+			o1 = obj[i] = new Spinner( SPINNERSHAPE, M_BIG, playerShip );
+			((Spinner *)o1)->newShip();
+			((Spinner *)o1)->setFpos( o2->VFx, o2->VFy );
 			}
 		else
 			{
-			obj[i]->shape = ASTSHAPE3;
-			obj[i]->setBitmap(Gbit[SMALLAST]);
-			obj[i]->Fmass = M_SMALL;
+			o1->shape = ASTSHAPE3;
+			o1->setBitmap(Gbit[SMALLAST]);
+			o1->Fmass = M_SMALL;
 			numasts++;
 			}
 
 		blastpair(na, i);
-		obj[i]->explode(20,10);
+		GboomList.add( o1->VFx, o1->VFy, 20, 10, o1->VFxvel, o1->VFyvel);
 		numasts = numasts + 2;
 		upscore(killer, 50);
 		}
-	else if (obj[i]->shape == ASTSHAPE3)
-		{	
-		obj[i]->explode(10, 8);
-		obj[i]->die(); numasts--; upscore(killer, 100);
+	else if (o1->shape == ASTSHAPE3)
+		{
+		GboomList.add( o1->VFx, o1->VFy, 10, 8, o1->VFxvel, o1->VFyvel);
+		o1->die(); numasts--; upscore(killer, 100);
+
 		}
-	else if (obj[i]->shape == SPINNERSHAPE)
+	else if (o1->shape == SPINNERSHAPE)
 		{	
 		// it takes 3 hits to kill a spinner
-		obj[i]->explode(10, 8);
-		((Spinner *)obj[i])->VhitCount++;
-		if ( ((Spinner *)obj[i])->VhitCount > 3)
+		GboomList.add( o1->VFx, o1->VFy, 10, 8, o1->VFxvel, o1->VFyvel);
+		((Spinner *)o1)->VhitCount++;
+		if ( ((Spinner *)o1)->VhitCount > 3)
 			{
 			upscore(killer, 200);
 			delete obj[i];
@@ -454,9 +460,9 @@ void killast(int killer, int i)
 		}
 	else	// enemy {ship or bullet}	
 		{
-		//Sound::play_file("boom.voc");
-		obj[i]->explode(9, 7);
-		obj[i]->die(); upscore(killer, 700);
+
+		GboomList.add( o1->VFx, o1->VFy, 9, 7, o1->VFxvel, o1->VFyvel);
+		o1->die(); upscore(killer, 700);
 		}
 	}
 
@@ -471,7 +477,7 @@ void drawobjs()
 		obj[i]->draw();
 		}
 		
-	Boom::draw();
+	GboomList.draw();
 	}	
 
 
@@ -480,7 +486,7 @@ int moveobjs()
 	{	
 	int i, j, crash=0;	// Indexes
 
-	Boom::tick();	// pass time for explosions
+	GboomList.tick();	// pass time for explosions
 
 	// tell each object to pass 1 frame of time
 
@@ -508,8 +514,11 @@ int moveobjs()
 			else
 				{	
 				crash = 2;
-				//Sound::play_file("crash.voc");
-				playerShip.explode(BMAX-1, 50);
+				GboomList.add( playerShip.VFx, 
+					   playerShip.VFy, 
+					   BMAX-1, 40, 
+					   playerShip.VFxvel, 
+					   playerShip.VFyvel);
 				playerShip.die();
 				killast(SHIP, i);
 				playerShip.reset();
@@ -549,7 +558,6 @@ void fire()
 	obj[nextbul]->setTime(30);	// loops before bullet expires	
 	nextbul++; if (nextbul == LASTBUL+1) nextbul = FBUL;
 
-	//Sound::play_file("shotgun.voc");
 	}
 
 // return 1=easy, 4=medium, 8=hard
@@ -606,7 +614,7 @@ void displayScreen(const char *file)
 	scn.put(x,y);
 
 	}
-	
+
 void showInfo() 
 	{
 	displayScreen("BACK.RAW");
@@ -620,7 +628,7 @@ void showInfo()
 
 	gl_write( x+10, y+20, "TO PLAY THE GAME");
 	gl_write( x+10, y+30, "");
-	gl_write( x+10, y+40, "  KEY        ACTION");
+	gl_write( x+10, y+40, " KEY         ACTION");
 	gl_write( x+10, y+50, " ----------  -----------------------");
 	gl_write( x+10, y+60, " LEFT ARROW  TURN SHIP LEFT");
 	gl_write( x+10, y+70, " RIGHT ARRW  TURN SHIP RIGHT");
@@ -629,8 +637,37 @@ void showInfo()
 	gl_write( x+10, y+100," DOWN ARROW  SHIELD, ONE PER SHIP");
 	gl_write( x+10, y+110," LEFT ALT    HYPERSPACE (ZAP TO");
 	gl_write( x+10, y+120,"             RANDOM AREA ON SCREEN)");
-	gl_write( x+10, y+130," P           PAUSE");
-	gl_write( x+10, y+140," Q           QUIT GAME");
+	gl_write( x+10, y+140," P           PAUSE (YOU CAN SWITCH");
+	gl_write( x+10, y+150,"             VC's WHILE PAUSED)");
+	gl_write( x+10, y+160," Q           QUIT GAME");
+
+	gl_write( x+40, y+190,"(Press any key for scoring info)");
+	gl_setwritemode(WRITEMODE_OVERWRITE);
+
+	};
+
+void showScoringInfo() 
+	{
+	displayScreen("BACK.RAW");
+	
+	int x = (WIDTH - 320)>>1;
+	int y = (HEIGHT - 200)>>1;
+
+	gl_setwritemode(WRITEMODE_MASKED);
+
+	gl_write( x+120, y+0,"SASTEROIDS");
+
+	gl_write( x+10, y+20, "SCORING INFORMATION");
+	gl_write( x+10, y+30, "");
+	gl_write( x+10, y+40, " POINTS      WHAT");
+	gl_write( x+10, y+50, " ----------  -----------------------");
+	gl_write( x+10, y+60, " 25          Big Asteroid");
+	gl_write( x+10, y+70, " 50          Medium Asteroid");
+	gl_write( x+10, y+80, " 100         Small Asteroid");
+	gl_write( x+10, y+100, " 200         Spinning Alien");
+	gl_write( x+10, y+110," 700         Shooting Alien or Bullet");
+	gl_write( x+10, y+130," 500*level   Medium Asteroid Alone");
+	gl_write( x+10, y+140," 1000*level  Big Asteroid All Alone");
 	gl_write( x+6, y+160,"Send bug reports, patches, sounds");
 	gl_write( x+6, y+170,"or cool bitmaps to pitzel@cs.sfu.ca");
 
@@ -781,14 +818,14 @@ int setGameKeyboard() return result;
 // play a game..
 void playGame()
 {
+	int backOff=0;
 	int	showTimings=0,flash=0;
 	char	text[70];		// for showTimings mode
-		
 	int pause = 0, i;
 
 	char GameOver = 0;
 	
-	Boom::die();
+	GboomList.die();
 
 	playerShip.ships(3);	// start with 3 ships
 	score = 0; oldscore = -1;
@@ -936,7 +973,7 @@ void playGame()
 		    		{
 		    		Keyboard::clearhit();
 				playerShip.revive();
-				Boom::die();
+				GboomList.die();
 				}
 		    	}
 		    	
@@ -944,7 +981,7 @@ void playGame()
 		    		{
 		    		Keyboard::clearhit();
 				playerShip.revive();
-				Boom::die();
+				GboomList.die();
 		    		}
 		     }
 
@@ -1019,8 +1056,9 @@ void playGame()
 
 			
 #ifdef BACKDROPS
-		memcpy(VBUF,Gbackdrop.map(),64000);
-//		gl_putbox(0,0,320,200,Gbackdrop.map());
+		gl_putbox(0,0,320,200,Gbackdrop.map()+backOff);
+		//backOff +=320;
+		//if (backOff==64000) backOff=0;
 #else			
 		gl_clearscreen(0);
 #endif
@@ -1032,7 +1070,7 @@ void playGame()
 		else
 			{
 			// time how long to create one frame of animation
-			sprintf(text,"usec per frame: %d",  Timer::sync() );
+			sprintf(text,"usec per frame: %d   %d",  Timer::sync(), numasts );
 			gl_write(0, HEIGHT-Ui::fontHeight(), text);
 			}
 
@@ -1052,7 +1090,7 @@ int main(int argc, char **argv)
 		exit(-1);
 		}
 
-	HiScore	scores("sast.scores");
+	HiScore	scores(SCOREDIR"sast.scores");
 
 	cout << "Initializing game." << endl;
 
@@ -1078,7 +1116,6 @@ int main(int argc, char **argv)
 	Ship::init();	// setup up player's ship
 	Spinner::init();
 	
-
 	Timer::setFrameRate( FPS );
 
 	uid_t cur_uid=getuid(); // init vga, ui stuff
@@ -1086,13 +1123,15 @@ int main(int argc, char **argv)
 	Ui::init();		// but we want to keep euid as root for      
 	setreuid(cur_uid,-1);	// writing to the hi-score file, so we do this
 				// little bit of uid save/restoring.   
+	Ui::setFontColor(WHITE);
+	
+	// determines at what point sprites will wrap-around
+	Trajectory::setFwrap( INT2FP(WIDTH), INT2FP(HEIGHT) );
 
 	int done=0;
 
-	//Sound::init();
-	
 	showTitle();
-	
+
     while(!done)
         {
 	switch(getchar())
@@ -1108,6 +1147,8 @@ int main(int argc, char **argv)
 		case 'I':
 			showInfo();
 			getchar();
+			showScoringInfo();
+			getchar();
 			showTitle();
 			break;
 			
@@ -1118,7 +1159,6 @@ int main(int argc, char **argv)
         		// if score was good enough, add to high score list
         		if (scores.isHigh(score) != -1)
         			{
-        			//Sound::play_file("applause.voc");
         			scores.add( getHighScoreName() , score );
         			showScores(scores);
 				getchar();
@@ -1137,7 +1177,6 @@ int main(int argc, char **argv)
 
 	// shutdown & restore text mode
 	Ui::restore();
-	//Sound::restore();
-
+	
 	return 0;
 }
